from random import random
from poplib import POP3
from StringIO import StringIO
from string import split,join,replace

from Globals import InitializeClass, DTMLFile
from DateTime import DateTime
from AccessControl.Permissions import view_management_screens
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName, _dtmldir


import email.Message
import email.Parser
import email.Header

class MailaddTool(UniqueObject, SimpleItem, PropertyManager, ActionProviderBase):
    """
    A simple mail add tool for CMF site 
    """

    id = 'portal_mailadd'
    meta_type = 'CMF Mailadd Tool'
    security = ClassSecurityInfo()
    manage_options=( 
                    ({ 'label' : 'Configuration', 'action' : 'manage_propertiesForm' },
                    { 'label' : 'Overview', 'action' : 'manage_overview' },) +
                    ActionProviderBase.manage_options +
                    PropertyManager.manage_options +
                    SimpleItem.manage_options)
    
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile( 'dtml/explainMailadd', globals())

    def __init__(self):
        self._setProperty('folder', '', 'string')
        self._setProperty('file_folder', '', 'string')
        self._setProperty('default_content_type', 'Document', 'string')
        self._setProperty('mailadd_host', 'localhost', 'string')
        self._setProperty('mailadd_user', '', 'string')
        self._setProperty('mailadd_passwd', '', 'string')
        self._setProperty('mailadd_email_addr', '', 'string')
        self._setProperty('workflow_action', '', 'string')
        self._setProperty('body_separater', '', 'string')

    def _getMemberFolder(self,id):
        """
        Return the member object
        """
        mtool = getToolByName(self, 'portal_membership')
        
        return mtool.getHomeFolder(id=id)

    def _setWorkFlow(self,obj,action):
        """
        Return the member object
        """
        wtool = getToolByName(self, 'portal_workflow')
        wtool.doActionFor(obj,action)
        return None

    def _searchMembersEmail(self,email):
        """
        Search the member via email address
        """
        if email:
            email = email.strip().lower()
        if not email:
            email = None


        mtool = getToolByName(self, 'portal_membership')
        md = getToolByName(self,'portal_memberdata')

        res = []
        for member in mtool.listMembers():
            #user = md.wrapUser(u)
            u = member.getUser()
            if email:
                if member.getProperty('email').lower().find(email) == -1:
                    continue
            res.append(member)
        return res

    def _generateId(self):
        """
        Generate unigue Id
        """
        now=DateTime()
        time='%s.%s' % (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
        rand=str(random())[2:6]

        return time+rand

    def _PartsRecurse(self,msg):
        """
        Get the multiparts of the mail
        """
        global mailparts
        if msg.is_multipart():
            for subMsg in msg.get_payload():
                self._PartsRecurse(subMsg)
        else:    
            name=msg.get_param("name")
            type=msg.get_content_type()
            data=StringIO(msg.get_payload(decode=1))
            mailparts.append({"type":type,"data":data,"name":name}) 
        return None

    def _getSenderMail(self,mailfrom):
        """
        Get the sender email for checking
        """
        sendl = mailfrom.split()
        send =''
        for s in sendl:
            if s.find('@') != -1:
                send = s
                if send[0]=="<":
                    send = send[1:]
               
                if send[-1:]==">":
                    send = send[:-1]
                break
                
        return send

    def _subUnicode(self,subject=''):
        """
        Enocde the mail subject to some encoding, mainly utf-8
        """
        subject_list = email.Header.decode_header(subject)
        rs=''
        s_encoding=''
        for s, encoding in subject_list:
            if encoding:
                s_encoding=encoding
                rs = rs + unicode(s, encoding)
            else:
                rs = rs + unicode(s)
        if s_encoding:
            return rs.encode(s_encoding)
        else:
            return rs
    
    security.declareProtected(ManagePortal, 'receice')
    def receive(self):
        """Receive mails
        """
        global mailparts
        mailparts=[]
        #create a POP3 client instance
        m = POP3(self.getProperty("mailadd_host"))
        #Authenticate

        m.user(self.getProperty("mailadd_user"))
        m.pass_(self.getProperty("mailadd_passwd"))

        #get number of total message
        total = len(m.list()[1])

        #get uid list
        l = m.uidl()[1]
        #make valid uid list
        suidls = []
        for uidline in l:
            ut = split(uidline," ")
            if len(ut) > 0:
                suidls.append(ut[len(ut)-1])
            else:
                suidls.append(uidline)

        if not hasattr(self,"uidls"):
            self.uidls = []
            self._p_changed = 1

        cnt = 1
        uidl_changed = 0
        for uid in suidls:
            if uid not in self.uidls:
                #New mail arrived...
                ml = m.retr(cnt)[1]
                buf = "\n".join(ml)
                mail = email.Parser.Parser().parsestr(buf)
                self.add_mail_entry(mail)
                uidl_changed = 1
            cnt = cnt + 1
        if uidl_changed:
            self.uidls = suidls
            self._p_changed = 1

        m.quit()

    security.declarePrivate('add_mail_entry')
    def add_mail_entry(self,mail):
        """
        Add the mail content to member folder
        """
        #add entry from mail
        #If image was attached, 
        global mailparts
        w_action = self.getProperty("workflow_action")
        mailparts= []
        self._PartsRecurse(mail)
        bd = ""
        pd = []
        fd = []
        mainpart = 0
        bdbuf = None
        for d in mailparts:
            if mainpart == 0 and d["type"].find("text") != -1:
                #main part found
                mainpart = 1
                bdbuf = d["data"]
            elif d["type"].find("image") != -1:
                pd.append(d)
            else:
                fd.append(d)
            
        if not bdbuf:
            return
        #check sender
        sender = self._getSenderMail(mail.get('From'))
        allowed_email = self.getProperty("mailadd_email_addr")
        
        if allowed_email:
            #check sender.
            go = 0
            ae_list = split(allowed_email,",")
            for addr in ae_list:
                if sender.find(addr) != -1:
                        go = 1
                        break;
                if go:
                    break
            if not go:
                #sender was mismatched....
                return
        #check the member home 
        member_list =  self._searchMembersEmail(addr)
        if len(member_list)>0:
            m_folder = self._getMemberFolder(member_list[0].id) 

        else:
            # No member folder, let's finish
            return
            
        if m_folder:
            # have this member folder
            body_l = []
            separater = self.getProperty("body_separater")
            cnt = 0
            lines = bdbuf.readlines()
            for line in lines:
                line = replace(line,"\n","")
                line = replace(line,"\r","")
                if line and ord(line[0]) == 32:
                    #remove first space
                    line = line[1:]
                if len(separater) > 0 and line == separater:
                    break
                body_l.append(line)
                cnt = cnt + 1
            
            #add this mail to Plone :-) 

            file_folder = self.file_folder
            pict_attached = 0
            pictid = ""

            if len(self.folder) > 0 and  self.folder not in m_folder.contentIds():

                m_folder.invokeFactory('Folder', self.folder)
                add_folder = getattr(m_folder,self.folder,None)
            elif len(self.folder) > 0:
                add_folder = getattr(m_folder,self.folder,None)
            else:
                add_folder = m_folder
            
            if len(self.file_folder) > 0 and  self.file_folder not in m_folder.contentIds():

                m_folder.invokeFactory('Folder', self.file_folder)
                file_folder = getattr(m_folder,self.file_folder,None)
            elif  len(self.file_folder) > 0:
                file_folder = getattr(m_folder,self.file_folder,None)
            else:
                file_folder = m_folder
            
            if len(pd) > 0:
                #try adding a image
                for pic in pd:
                    if len(pic["name"]) > 0:
                        img_id = pic["name"]
                        file_folder.invokeFactory('Image',img_id)
                        image = getattr(file_folder,img_id)
                        try:
                            image.update_data(pic["data"].getvalue())
                        except AttributeError:
                            # we probably have ATCT's Image type
                            image.setImage(pic["data"].getvalue())

            if len(fd) > 0:
                #try adding a file
                for fp in fd:
                    if len(fp["name"]) > 0:
                        file_id = fp["name"]
                        file_folder.invokeFactory('File',file_id)
                        ff = getattr(file_folder,file_id)
                        try:
                            ff.update_data(fp["data"].getvalue())
                        except:
                            pass
                            
            #add entry body
            if len(body_l) > 1:
                entry_title = self._subUnicode(mail.get("Subject"))
                entry_body = join(body_l,"\n")
                entry_id = self._generateId()
                add_folder.invokeFactory(self.default_content_type, entry_id)
                try:
                    entry = getattr(add_folder,entry_id)
                    entry.setTitle(entry_title)
                    entry.edit(text_format='structured-text',text=entry_body)
                    if  w_action:
                        self._setWorkFlow(entry,w_action)
                except:
                    pass
                    
InitializeClass(MailaddTool)
